//+------------------------------------------------------------------+
//|                                                     Brent EA.mq5 |
//|                                  Copyright 2024, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2024, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"

//Libraries
#include  <Trade\Trade.mqh>
CTrade ExtTrade;
#include <TrailingStop\ATRTrailingStop3.mqh>
ATRTrailingStop ExtATRTrailingStop;

//Inputs
input double atr_multiple = 5.0;
input double lot_multiple = 1.0;
input double profit_target = 10;
input double max_loss = 20;
input int position_size = 2;

//Set this value between 0 and 1 to control how much data is used
double consumption = 0.01;
//We want to know which symbol has the least number of bars.
double brent_bars = (double) NormalizeDouble((iBars("UK Brent Oil",PERIOD_CURRENT) * consumption),0);
double wti_bars = (double) NormalizeDouble((iBars("WTI_OIL",PERIOD_CURRENT) * consumption),0);
//Select the lowest
double max_bars = (brent_bars < wti_bars) ? brent_bars : wti_bars;
//How far into the future are we forecasting
double look_ahead = NormalizeDouble((max_bars / 4),0);
//How many bars should we fetch? 
int fetch = (int) (max_bars - look_ahead) - 1;
//Matrix A stores our inputs. y is the output. x is the coefficients. 
matrix A = matrix::Zeros(fetch,6);
matrix y = matrix::Zeros(fetch,1);
vector wti_price = vector::Zeros(fetch);
vector brent_price = vector::Zeros(fetch);
vector spread;
vector intercept = vector::Ones(fetch);
matrix x = matrix::Zeros(6,1);
double forecast = 0;
double ask = 0;
double bid = 0;
double min_volume = 0;

string brent = "UK Brent Oil";
string wti = "WTI_OIL";
bool model_initialized = false;
int OnInit()
  {
  //Initialise trailing stops
   if(atr_multiple > 0) ExtATRTrailingStop.Init(atr_multiple);
   min_volume = SymbolInfoDouble(brent,SYMBOL_VOLUME_MIN);
  return(INIT_SUCCEEDED);
//---
   
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   ask = SymbolInfoDouble(brent,SYMBOL_ASK);
   bid = SymbolInfoDouble(brent,SYMBOL_BID);
   if(model_initialized)
      {
         if(PositionsTotal() == 0)
            {
                  forecast = 0;
                  forecast = ModelForecast();
                  InterpretForecast();
            }
         
         else
            {
                  ManageTrades();
            }
      }
   
   else
      {
         model_initialized = InitializeModel();
      }
   
  }
//+------------------------------------------------------------------+

void ManageTrades()
   {
      if(AccountInfoDouble(ACCOUNT_PROFIT) > profit_target)  CloseAll(); 
      if(AccountInfoDouble(ACCOUNT_PROFIT) < (-1 * max_loss))  CloseAll();
   }

void InterpretForecast()
   {
      if(forecast != 0)
         {
            if(forecast > iClose(_Symbol,PERIOD_CURRENT,0))
                     {
                        check_buy();
                     }
                     
            else if(forecast < iClose(_Symbol,PERIOD_CURRENT,0))
                     {
                        check_sell();
                     }   
         }
   }

void check_buy(){
      if(PositionsTotal() == 0){
                        for(int i = 0; i < position_size;i++){
                                 ExtTrade.Buy(lot_multiple * min_volume,brent,ask,0,0,"BUY");
                        }
               }
}

void check_sell(){
      if(PositionsTotal() == 0){
                           for(int i = 0; i < position_size;i++){
                                     ExtTrade.Sell(lot_multiple * min_volume,brent,bid,0,0,"SELL");
                           }
      }
}

void CloseAll(void)
{
    for(int i=PositionsTotal()-1; i>=0; i--)
    {
        if(PositionSelectByTicket(PositionGetTicket(i)))
        {
            if(PositionGetSymbol(i) == brent)
            {
                  ulong ticket;
                  ticket = PositionGetTicket(i);
                  ExtTrade.PositionClose(ticket);
            }
        }
    }
}

void close_buy(){
      ulong ticket;
      int type;
            if(PositionsTotal() > 0){
                  for(int i = 0; i < PositionsTotal();i++){
                                    ticket = PositionGetTicket(i);
                                    type = (int)PositionGetInteger(POSITION_TYPE);
                                             if(type == POSITION_TYPE_BUY){
                                                    ExtTrade.PositionClose(ticket);
                                    }
                  }
            }
}

void close_sell(){
      ulong ticket;
      int type;
            if(PositionsTotal() > 0){
                  for(int i = 0; i < PositionsTotal();i++){
                                    ticket = PositionGetTicket(i);
                                    type = (int)PositionGetInteger(POSITION_TYPE);
                                             if(type == POSITION_TYPE_SELL){
                                                    ExtTrade.PositionClose(ticket);
                                    }
                  }
            }
}


bool InitializeModel()
   {
      //Try select the symbols
      if(SymbolSelect(brent,true) && SymbolSelect(wti,true))
      {
         Print("Symbols Available. Bars: ",max_bars," Fetch: ",fetch," Look ahead: ",look_ahead);
         //Get historical data on Brent , our model output
         y.CopyRates(brent,PERIOD_CURRENT,COPY_RATES_CLOSE,1,fetch);
         //model input
         A.CopyRates(brent,PERIOD_CURRENT,COPY_RATES_CLOSE,(1 + look_ahead),fetch);
         brent_price.CopyRates(brent,PERIOD_CURRENT,COPY_RATES_CLOSE,(1+look_ahead),fetch);
         wti_price.CopyRates(wti,PERIOD_CURRENT,COPY_RATES_CLOSE,(1+look_ahead),fetch);
         //Calculate the spread
         spread = brent_price - wti_price;
         Print("The Current Spread: ",spread);
         A.Reshape(3,fetch);
         //Add the spread to the input matrix
         A.Row(spread,1);
         //Add a column for the intercept
         A.Row(intercept,2);
         //Reshape the matrices
         A = A.Transpose();
         y = y.Transpose();
         //Inspect the matrices
         if((A.Cols() == 0 || y.Cols() == 0))
            {
               Print("Error occured when copying historical data");
               Print("A rows: ",A.Rows()," y rows: ",y.Rows()," A columns: ",A.Cols()," y cols: ",y.Cols()); 
               Print("A");
               Print(A);
               Print("y");
               Print(y);
               return(false);
            }
            
         else
            {
               Print("No errors occured when copying historical data");
               x = A.PInv().MatMul(y);
               Print("Finished Fitting The Model");
               Print(x);
               return(true);
            }
      }
      
      Print("Faield to select symbols");
      return(false);
   }
   
double ModelForecast()
   {
      if(model_initialized)
         {
            //model input
            A.CopyRates(brent,PERIOD_CURRENT,COPY_RATES_CLOSE,0,1);
            brent_price.CopyRates(brent,PERIOD_CURRENT,COPY_RATES_CLOSE,0,1);
            wti_price.CopyRates(wti,PERIOD_CURRENT,COPY_RATES_CLOSE,0,1);
            //Calculate the spread
            spread = brent_price - wti_price;
            Print("The Spread: ",spread);
            A.Reshape(3,fetch);
            //Add the spread to the input matrix
            A.Row(spread,1);
            //Add a column for the intercept
            A.Row(intercept,2);
            //Reshape the matrices
            A = A.Transpose();
            double _forecast = (A[0][0]*x[0][0]) + (A[1][0]*x[1][0]) + (A[2][0]*x[2][0]);
            return(_forecast);
         }
         return(0);
   }